@@ -40,24 +40,17 @@
 #' @param filterHWE [logical] Set to exclude markers that fail HWE exact test
 #' (via \code{\link{check_hwe}} or \code{\link{perMarkerQC}}). Requires hweTh to
 #' be set.
-#' @param hweTh [double] Significance threshold for deviation from HWE.
 #' @param filterMAF [logical] Set to exclude markers that fail minor allele
 #' frequency or minor allele count threshold (via \code{\link{check_maf}} or
 #' \code{\link{perMarkerQC}}). Requires mafTh or macTh to be set.
-#' @param mafTh [double] Threshold for minor allele frequency cut-off.
-#' @param macTh [double] Threshold for minor allele cut cut-off, if both mafTh
-#' and macTh are specified, macTh is used (macTh = mafTh\*2\*NrSamples).
 #' @param filterSNPMissingness [logical] Set to exclude markers that have
 #' excessive missing rates across samples (via
 #' \code{\link{check_snp_missingness}} or \code{\link{perMarkerQC}}). Requires
 #' lmissTh to be set.
-#' @param lmissTh [double] Threshold for acceptable variant missing rate across
-#' samples.
-#' @param path2plink [character] Absolute path to directory where external plink
-#' software \url{https://www.cog-genomics.org/plink/1.9/} can be found, i.e.
-#' plink should be accesible as path2plink/plink -h. If not
-#' provided, assumed that PATH set-up works and plink will be found by
-#' system("plink").
+#' @inheritParams checkPlink
+#' @inheritParams check_maf
+#' @inheritParams check_hwe
+#' @inheritParams check_snp_missingness
 #' @param verbose [logical] If TRUE, progress info is printed to standard out.
 #' @param showPlinkOutput [logical] If TRUE, plink log and error messages are
 #' printed to standard out.
@@ -96,15 +89,15 @@ cleanData <- function(indir, name, qcdir=indir,
                       filterAncestry=TRUE, filterRelated=TRUE,
                       filterSNPMissingness=TRUE, lmissTh=0.01,
                       filterHWE=TRUE, hweTh=1e-5,
-                      filterMAF=TRUE, macTh=20, mafTh=0.01,
+                      filterMAF=TRUE, macTh=20, mafTh=NULL,
                       path2plink=NULL, verbose=FALSE,
                       showPlinkOutput=TRUE) {
     sampleFilter <- c(filterAncestry, filterRelated, filterSex,
                       filterHeterozygosity, filterSampleMissingness)
     markerFilter <- c(filterHWE, filterMAF, filterSNPMissingness)
 
-    prefix <- paste(indir,"/", name, sep="")
-    out <- paste(qcdir,"/", name, sep="")
+    prefix <- makepath(indir, name)
+    out <- makepath(qcdir, name)
 
     if (!any(c(sampleFilter, markerFilter))) {
         stop("No per-sample and per-marker filters chosen")
@@ -117,25 +110,26 @@ cleanData <- function(indir, name, qcdir=indir,
         message("No per-marker filter chosen, carry on with removing samples ",
                 "that fail per-samples QCs")
     }
-    if (!file.exists(paste(prefix, ".fam",sep=""))){
-        stop("plink family file: ", prefix, ".fam does not exist.")
-    }
-    if (!file.exists(paste(prefix, ".bim",sep=""))){
-        stop("plink snp file: ", prefix, ".bim does not exist.")
-    }
-    if (!file.exists(paste(prefix, ".bed",sep=""))){
-        stop("plink binary file: ", prefix, ".bed does not exist.")
-    }
+    checkFormat(prefix)
+    path2plink <- checkPlink(path2plink)
+
     # Remove remove.IDs file if already existing
-    system(paste("rm -f ", out, ".remove.IDs", sep="" ))
+    removeIDs <- NULL
     if (any(sampleFilter)) {
         if (filterRelated) {
             if (!file.exists(paste(out, ".fail-IBD.IDs", sep=""))){
                 stop("filterRelated is TRUE but file ", out,
                      ".fail-IBD.IDs does not exist")
             } else {
-                system(paste("cat ", out, ".fail-IBD.IDs >> ",
-                             out, ".remove.IDs", sep=""))
+                if (verbose) {
+                    message("Read individual IDs that failed relatedness check")
+                }
+                removeIDs <- rbind(removeIDs,
+                                   data.table::fread(paste(out, ".fail-IBD.IDs",
+                                                     sep=""),
+                                                     data.table=FALSE,
+                                                     stringsAsFactors=FALSE,
+                                                     header=FALSE))
             }
         }
         if (filterAncestry) {
@@ -143,8 +137,16 @@ cleanData <- function(indir, name, qcdir=indir,
                 stop("filterAncestry is TRUE but file ", out,
                      ".fail-ancestry.IDs does not exist")
             } else {
-                system(paste("cat ", out, ".fail-ancestry.IDs >> ",
-                             out, ".remove.IDs", sep=""))
+                if (verbose) {
+                    message("Read individual IDs that failed ancestry check")
+                }
+                removeIDs <- rbind(removeIDs,
+                                   data.table::fread(paste(out,
+                                                           ".fail-ancestry.IDs",
+                                                           sep=""),
+                                                     data.table=FALSE,
+                                                     stringsAsFactors=FALSE,
+                                                     header=FALSE))
             }
         }
         if (filterHeterozygosity) {
@@ -152,8 +154,16 @@ cleanData <- function(indir, name, qcdir=indir,
                 stop("filterHeterozygosity is TRUE but file ", out,
                      ".fail-het.IDs does not exist")
             } else {
-                system(paste("cat ", out, ".fail-het.IDs >> ",
-                             out, ".remove.IDs", sep=""))
+                if (verbose) {
+                    message("Read individual IDs that failed heterozygosity ",
+                            "check")
+                }
+                removeIDs <- rbind(removeIDs,
+                                   data.table::fread(paste(out, ".fail-het.IDs",
+                                                           sep=""),
+                                                     data.table=FALSE,
+                                                     stringsAsFactors=FALSE,
+                                                     header=FALSE))
             }
         }
         if (filterSampleMissingness) {
@@ -161,8 +171,16 @@ cleanData <- function(indir, name, qcdir=indir,
                 stop("filterSampleMissingness is TRUE but file ", out,
                      ".fail-imiss.IDs does not exist")
             } else {
-                system(paste("cat ", out, ".fail-imiss.IDs >> ",
-                             out, ".remove.IDs", sep=""))
+                if (verbose) {
+                    message("Read individual IDs that failed missingness check")
+                }
+                removeIDs <- rbind(removeIDs,
+                                   data.table::fread(paste(out,
+                                                           ".fail-imiss.IDs",
+                                                           sep=""),
+                                                     data.table=FALSE,
+                                                     stringsAsFactors=FALSE,
+                                                     header=FALSE))
             }
         }
         if (filterSex) {
@@ -170,29 +188,36 @@ cleanData <- function(indir, name, qcdir=indir,
                 stop("filterSex is TRUE but file ", out,
                      ".fail-sexcheck.IDs does not exist")
             } else {
-                system(paste("cat ", out, ".fail-sexcheck.IDs >> ",
-                             out, ".remove.IDs", sep=""))
+                if (verbose) {
+                    message("Read individual IDs that failed sex check")
+                }
+                removeIDs <- rbind(removeIDs,
+                                   data.table::fread(paste(out,
+                                                           ".fail-sexcheck.IDs",
+                                                           sep=""),
+                                                     data.table=FALSE,
+                                                     stringsAsFactors=FALSE,
+                                                     header=FALSE))
             }
         }
         # ensure unique IDs in remove.IDs
-        removeIDs <- data.table::fread(paste(out, ".remove.IDs",
-                                             sep=""),
-                                       header=FALSE,
-                                       data.table=FALSE, stringsAsFactors=FALSE)
         removeIDs <- removeIDs[!duplicated(removeIDs),]
+        if (verbose) message("Write file with remove IDs")
         write.table(removeIDs, paste(out, ".remove.IDs", sep=""),
                     col.names=FALSE, row.names=FALSE, quote=FALSE)
         keepIDs <- data.table::fread(paste(prefix, ".fam", sep=""),
                                      data.table=FALSE, stringsAsFactors=FALSE,
                                      header=FALSE)
-        keepIDs <- keepIDs[keepIDs[,2] %in% removeIDs[,2],1:2]
-        removeString <- paste(" --remove ", out, ".remove.IDs", sep="")
+        keepIDs <- keepIDs[!keepIDs[,2] %in% removeIDs[,2],1:2]
+        remove <- c("--remove", paste(out, ".remove.IDs", sep=""))
+        fail_samples <- nrow(removeIDs)
     } else {
         keepIDs <- data.table::fread(paste(prefix, ".fam", sep=""),
                                      data.table=FALSE, stringsAsFactors=FALSE,
                                      header=FALSE)
         removeIDs <- NULL
-        removeString <- ""
+        remove <- NULL
+        fail_samples <- 0
     }
 
     hwe <- NULL
@@ -203,50 +228,48 @@ cleanData <- function(indir, name, qcdir=indir,
         if (is.null(hweTh)) {
             stop("filterHWE is TRUE but hweTh not specified")
         } else {
-           hwe <- paste(" --hwe", hweTh)
+           hwe <- c("--hwe", hweTh)
         }
     }
     if (filterMAF) {
         if (is.null(mafTh) && is.null(macTh)) {
             stop("filterMAF is TRUE but neither mafTh or macTh are provided")
         }
-        if(!is.null(macTh)) {
-            all_samples <-  R.utils::countLines(paste(prefix, ".fam",
-                                                      sep=""))
-            keep_samples <- as.numeric(all_samples) - nrow(removeIDs)
-            mafTh <- macTh/(2*keep_samples)
-        }
-        if (verbose) {
-            if (!is.null(mafTh) && !is.null(macTh)) {
+
+        all_samples <-  R.utils::countLines(paste(prefix, ".fam", sep=""))
+        keep_samples <- as.numeric(all_samples) - fail_samples
+
+        if (!is.null(mafTh) && !is.null(macTh)) {
+            if (verbose) {
                 message("Both mafTh and macTh provided, macTh=", macTh,
-                        " is used (which corresponds to a mafTh=",
-                        round(mafTh, 6), ")")
-            } else if (!is.null(mafTh)) {
-                message("The mafTh is ", mafTh)
-            } else {
+                        " is used (corresponds to mafTh=", round(mafTh, 6), ")")
+            }
+        } else if (!is.null(mafTh)) {
+            if(is.null(macTh)) macTh <- mafTh*(2*keep_samples)
+            if (verbose) {
+                message("The mafTh is ", mafTh, " which corresponds to a mcfTh=",
+                        macTh)
+            }
+        } else {
+            if(is.null(mafTh)) mafTh <- macTh/(2*keep_samples)
+            if (verbose) {
                 message("The macTh is ", macTh," which corresponds to a mafTh=",
-                        round(mafTh, 6), ")")
+                        round(mafTh, 6))
             }
         }
-        maf <- paste(" --maf", mafTh)
+        maf <- c("--maf", mafTh)
     }
     if (filterSNPMissingness) {
         if (is.null(lmissTh)) {
             stop("filterSNPMissingness is TRUE but lmissTh not specified")
         } else {
-            missing <- paste(" --geno", lmissTh)
+            missing <- c("--geno", lmissTh)
         }
     }
-
-    if (!is.null(path2plink)) {
-        path2plink <- paste(gsub("/$", "", path2plink), "/", sep="")
-    }
-    checkPlink(path2plink)
-    system(paste(path2plink, "plink --bfile ", prefix,
-                 removeString,
-                 maf, hwe, missing,
-                 " --make-bed --out ", out, ".clean",
-                 sep=""),
-           ignore.stdout=!showPlinkOutput, ignore.stderr=!showPlinkOutput)
+    if (verbose) message("Remove individual IDs and markers IDs that failed QC")
+    sys::exec_wait(path2plink,
+                   args=c("--bfile", prefix, remove, maf, hwe, missing,
+                          "--make-bed", "--out", paste(out, ".clean", sep="")),
+           std_out=showPlinkOutput, std_err=showPlinkOutput)
     return(list(passIDs=keepIDs, failIDs=removeIDs))
 }
